﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace ObservableCollectionSample
{
    /// <summary>
    /// Interaktionslogik für MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        ObservableCollection<Person> liste = new ObservableCollection<Person>();

        public MainWindow()
        {
            InitializeComponent();
            listBox1.ItemsSource = GetPersons();
            listBox1.DisplayMemberPath = "Name";
            listBox1.SelectedIndex = 0;
            listBox1.Focus();
        }

        private ObservableCollection<Person> GetPersons()
        {
            liste.Add(new Person { Name = "Schulze", Alter = 56, Adresse = "Bremen" });
            liste.Add(new Person { Name = "Schmidt", Alter = 21, Adresse = "Kiel" });
            liste.Add(new Person { Name = "Meier", Alter = 30, Adresse = "Bonn" });
            liste.Add(new Person { Name = "Schneider", Alter = 29, Adresse = "Ulm" });
            liste.Add(new Person { Name = "Müller", Alter = 26, Adresse = "Köln" });
            liste.Add(new Person { Name = "Fischer", Alter = 68, Adresse = "Bremen" });
            liste.Add(new Person { Name = "Klein", Alter = 73, Adresse = "Hamburg" });
            liste.Add(new Person { Name = "Miller", Alter = 64, Adresse = "Köln" });
            liste.Add(new Person { Name = "Abel", Alter = 77, Adresse = "Aachen" });
            liste.Add(new Person { Name = "Smith", Alter = 46, Adresse = "Berlin" });
            liste.Add(new Person { Name = "de Jong", Alter = 31, Adresse = "Essen" });
            liste.Add(new Person { Name = "Mayer", Alter = 31, Adresse = "Hannover" });
            return liste;
        }

        private void btnAdd_Click(object sender, RoutedEventArgs e)
        {
            txtName.Text = ""; txtAddress.Text = ""; txtAge.Text = "";
            txtName.Focus();
            btnAdd.IsEnabled = false;
            btnCancel.IsEnabled = true;
            btnSave.IsEnabled = true;
            btnDelete.IsEnabled = false;
            // Bindungen lösen
            BindingOperations.ClearBinding(txtName, TextBox.TextProperty);
            BindingOperations.ClearBinding(txtAddress, TextBox.TextProperty);
            BindingOperations.ClearBinding(txtAge, TextBox.TextProperty);
        }

        private void btnSave_Click(object sender, RoutedEventArgs e)
        {
            Person person = new Person { Name = txtName.Text, Adresse = txtAddress.Text };
            if (txtAge.Text == "") person.Alter = null;
            else person.Alter = (txtAge.Text).To<int>();
            liste.Add(person);

            btnAdd.IsEnabled = true;
            btnCancel.IsEnabled = false;
            btnSave.IsEnabled = false;
            btnDelete.IsEnabled = true;
            SetBinding();
        }

        private void btnCancel_Click(object sender, RoutedEventArgs e)
        {
            SetBinding();
            btnAdd.IsEnabled = true;
            btnDelete.IsEnabled = true;
            btnCancel.IsEnabled = false;
            btnSave.IsEnabled = false;
            listBox1.SelectedIndex = 0;
        }

        private void SetBinding()
        {
            // Bindungen neu festlegen
            Binding binding = new Binding("Name");
            txtName.SetBinding(TextBox.TextProperty, binding);

            binding = new Binding("Adresse");
            txtAddress.SetBinding(TextBox.TextProperty, binding);

            binding = new Binding("Alter");
            txtAge.SetBinding(TextBox.TextProperty, binding);
        }

        private void btnDelete_Click(object sender, RoutedEventArgs e)
        {
            int index = listBox1.SelectedIndex;
            liste.RemoveAt(index);
            if (liste.Count == 0) btnDelete.IsEnabled = false;
            else listBox1.SelectedIndex = 0;
        }
    }

    static class ExtensionMethods
    {
        public static T To<T>(this string text)
        {
            return (T)Convert.ChangeType(text, typeof(T));
        }
    }

    public class Person : INotifyPropertyChanged
    {
        // Ereignis
        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(PropertyChangedEventArgs e)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, e);
        }

        // Eigenschaften
        private string _Name;
        public string Name
        {
            get { return _Name; }
            set
            {
                _Name = value;
                OnPropertyChanged(new PropertyChangedEventArgs("Name"));
            }
        }

        private int? _Alter;
        public int? Alter
        {
            get { return _Alter; }
            set
            {
                _Alter = value;
                OnPropertyChanged(new PropertyChangedEventArgs("Alter"));
            }
        }

        private string _Adresse;
        public string Adresse
        {
            get { return _Adresse; }
            set
            {
                _Adresse = value;
                OnPropertyChanged(new PropertyChangedEventArgs("Adresse"));
            }
        }
    }
}

